;---------------------------------------------------------------------------------
; NAME:
;	dist_2points
;
; PURPOSE:
;	Returns distance in meters on a great circle route connecting two points on a sphere.
;
; CALLING SEQUENCE:
;	Result = Map_2Points(lon0, lat0, lon1, lat1)
;
; INPUTS:
;	Lon0, Lat0 = longitude and latitude of first point, P0 in degrees.
;	Lon1, Lat1 = longitude and latitude of second point, P1 in degrees.
;
; OUTPUTS:
;          Returns distance in meters
;
; MODIFICATION HISTORY:
; 	Written by:
;	  DMS, RSI	May, 2000. Written.
;         Darren Jackson  CIRES/ETL November, 2002 - Revised to be more efficient and
;                                                    perform only one task
;----------------------------------------------------------------------------------------

Function dist_2points, lon0, lat0, lon1, lat1

IF((lon0 eq lon1) and (lat0 eq lat1)) THEN BEGIN
   dist = 0.
   return, dist
ENDIF

k = !dpi/180.0
r_earth = 6378206.4d0 ;Earth equatorial radius, meters, Clarke 1866 ellipsoid

coslt1 = cos(k*lat1)
sinlt1 = sin(k*lat1)
coslt0 = cos(k*lat0)
sinlt0 = sin(k*lat0)
cosl0l1 = cos(k*(lon1-lon0))
cosc = sinlt0 * sinlt1 + coslt0 * coslt1 * cosl0l1  ;Cos of angle between pnts

return, acos(cosc) * r_earth
end
