;----------------------------------------------------------------
; NAME: harmonic_wave
;
; PURPOSE: Equation for harmonic wave.  Used by harm_tseries.pro
;
; LANGUAGE: IDL
;
; INPUTS:
;          name     type      dimension      description
;     -------------------------------------------------------
;          a        float     scalar         A harmonic coefficient for first three harmonics
;          b        float     scalar         B harmonic coefficient for first three harmonics
;          c        float     scalar         Constant for first three harmonics
;          w        float     scalar         frequency
;          n        long      scalar         total number of time points
;          dt       float     scalar         time increment
;
; OUTPUT:
;
; OUTPUT KEYWORDS:
;
;
; NOTES:
;        Period dictates how the climatological average is computed.
;        For a monthly time series, period = 12 will remove the annual
;        cycle from the anomaly time series.  period = 1 produces one
;        value for the climatology thus preserving the annual cycle.
;
; CREATION HISTORY:
;                    Darren Jackson CIRES/ETL June 2002
;-----------------------------------------------------------------------
FUNCTION harmonic_wave, a, b, c, w, n, dt

result = FLTARR(n)
FOR i = 0, n-1 DO BEGIN
   result(i) = a * COS(w * dt * i) + b * SIN(w * dt * i) + c
ENDFOR

RETURN, result
END
