;***********************************************
;*
;*   This program reads AMSRE SST NetCDF swath data from GHRSST 
;*   and extracts different fields for matchups with buoys.
;*
;*   Data is filtered for SST fill-values, land, and rain
;*   Fill value for anccillary data other than SST is -999.0
;*
;*   Output file is a binary structure (read with readu) of 10 fields
;*   with different length from orbit to orbit. 
;* 
;*   The fields are:
;*   1. jultme (dblarr), 2.scan line (intarr), 3.spot (intarr), 4.lat  5.lon  6.SST (fltarr)
;*   7. sst_bias (fltarr), 8.sst_std (fltarr), 9.wspd (fltarr), 10.dtwarm(fltarr) 
;*   11.rej_flag (byte)  12.conf_flag (byte), 13.proximity_flag(byte)
;*
;*   SLC 10.14.05
;*********************************************************
@../../../cdf2idl.pro
!path = '/homes/sandrac/idl_graphics:' + !path

nfill= -999.0

spawn,('ls -1 2004*.nc'),file_list   ;NetCDF data
print,file_list
totfiles=n_elements(file_list)
filestr=strarr(totfiles)
for k=0,totfiles-1 do begin

   print,file_list(k)
   filestrdum=string(file_list(k),"(A51)")
   temp=str_sep(filestrdum,'-')
   filestr(k)=strcompress(temp(0)+'_'+temp(1)+'_'+strmid(temp(5),0,3)+strmid(temp(4),8,11),/remove_all)

   ;******************************
   ; Extracting Year, Month, Day 
   ;******************************
   date=strcompress(temp(0),/remove_all)
   year=fix(strmid(date,0,4))
   month=fix(strmid(date,4,2))
   day=fix(strmid(date,6,2))
   jday=calday2jday(year,month,day)

   ;******************************
   ; opening file
   ;******************************
   ;read_amsrnc,file_list(k)
   ;cdf2idl,file_list(k),outfile='test',verbose='verbose'
   ncid = NCDF_OPEN(file_list(k)) 
   @test

   ;******************************
   ; Extracting Jultme array 
   ;******************************
   print,'Filename: ',filestrdum
   print,'Date, Start Date and Stop Date: ',date,' ',start_date,' ',stop_date
   print,'Start Time and Stop Time: ',start_time,' ',stop_time
   dumtme=str_sep(start_time,' ')
   hhmmss=dumtme(0)  ;hh:mm:ss
   dumtme2=str_sep(hhmmss,':')
   hour=dumtme2(0)
   minute=dumtme2(1)
   second=dumtme2(2)
   stmesec=hour*(60*60.d0)+minute*60.0d0+second*1d0 ;start tme in sec
   dtime=transpose(reform(sst_dtime))  ;time increment from start time(s)
   pxltme=stmesec+dtime   
   fractme=pxltme/(24*60*60.d0)
   jultme=jday+fractme

   ;******************************
   ;* SST (Kelvin)
   ;******************************
   scale_factor=float(strcompress(sea_surface_temperature_scale_factor,/remove_all))
   add_offset=float(strcompress(sea_surface_temperature_add_offset,/remove_all))
   sst_min=fix(strcompress(sea_surface_temperature_valid_min,/remove_all))
   sst_fill=long(strcompress(sea_surface_temperature__fillvalue,/remove_all))
   sstshort=reform(sea_surface_temperature)
   sst_bad=where(sstshort eq sst_min or sstshort eq sst_fill,cbsst) 
   sstf=scale_factor*sstshort+add_offset
   sstf(sst_bad)=nfill 
   sstf=transpose(sstf)
   ;pimage,sstf,mis_val=nfill,mis_color=1,vmin=280,vmax=305,$
   ;   /ps_port,/encapsulated,ps_file='amsr_sst.eps',title='SST',units='(K)'

   ;******************************
   ;* WIND 
   ;******************************
   wnd_scale_factor=float(strcompress(wind_speed_scale_factor,/remove_all))
   wnd_add_offset=float(strcompress(wind_speed_add_offset,/remove_all))
   ;wnd_min=fix(strcompress(wind_speed_valid_min,/remove_all))
   wnd=fix(reform(wind_speed))

   ;* converting bytes from 0-255 to -127 to 127
   wndnegbyt=where(wnd ge 128)
   wndsgnbyte=wnd
   wndsgnbyte(wndnegbyt)=wndsgnbyte(wndnegbyt)-255

   ;wnd_bad=where(wndsgnbyte eq wnd_min,cbwnd) 
   wndf=wnd_scale_factor*wndsgnbyte+wnd_add_offset
   ;wndf(wnd_bad)=nfill 
   wndf=transpose(wndf)
   ;pimage,wndf,mis_val=nfill,mis_color=1,vmin=0,vmax=20,$
   ;   /ps_port,/encapsulated,ps_file='amsr_wnd.eps',title='Wind Speed',units='(m/s)'

   ;******************************
   ;* DIURNAL AMPLITUDE (Kelvin)
   ;******************************
   dtw_scale_factor=float(strcompress(diurnal_amplitude_scale_factor,/remove_all))
   dtw_add_offset=float(strcompress(diurnal_amplitude_add_offset,/remove_all))
   ;dtw_min=fix(strcompress(diurnal_amplitude_valid_min,/remove_all))
   dtw=fix(reform(diurnal_amplitude))
   dtw=diurnal_amplitude

   ;* converting bytes from 0-255 to -127 to 127
   negbyt=where(dtw ge 128)
   dtwsgnbyte=dtw
   dtwsgnbyte(negbyt)=dtwsgnbyte(negbyt)-255

   ;dtw_bad=where(dtwsgnbyte eq dtw_min,cbdtw) 
   dtwf=dtw_scale_factor*dtwsgnbyte+dtw_add_offset
   ;dtwf(dtw_bad)=nfill 
   dtwf=transpose(dtwf)
   ;pimage,dtwf,mis_val=nfill,mis_color=1,vmin=0,vmax=2.5

   ;******************************
   ;* SSES 
   ;******************************
   sse_bias_scale_factor=float(strcompress(sses_bias_error_scale_factor,/remove_all))
   sse_bias_add_offset=float(strcompress(sses_bias_error_add_offset,/remove_all))
   ;sse_bias_min=fix(strcompress(sses_bias_error_valid_min,/remove_all))
   sse_bias=fix(reform(sses_bias_error))

   ;* converting bytes from 0-255 to -127 to 127
   sse_bias_negbyt=where(sse_bias ge 128)
   sse_bias_sgnbyte=sse_bias
   sse_bias_sgnbyte(sse_bias_negbyt)=sse_bias_sgnbyte(sse_bias_negbyt)-255

   ;sse_bias_bad=where(sse_bias_sgnbyte eq sse_bias_min,cbsse_bias) 
   sse_biasf=sse_bias_scale_factor*sse_bias_sgnbyte+sse_bias_add_offset
   ;sse_biasf(sse_bias_bad)=nfill 
   sse_biasf=transpose(sse_biasf)
   ;pimage,sse_biasf,mis_val=nfill,mis_color=1,vmin=-0.5,vmax=0.5,$
   ;   /ps_port,/encapsulated,ps_file='amsr_bias.eps',title='Bias',units='(Kelvin)'

   sse_std_scale_factor=float(strcompress(sses_standard_deviation_error_scale_factor,/remove_all))
   sse_std_add_offset=float(strcompress(sses_standard_deviation_error_add_offset,/remove_all))
   ;sse_std_min=fix(strcompress(sses_standard_deviation_error_valid_min,/remove_all))
   sse_std=fix(reform(sses_standard_deviation_error))

   ;* converting bytes from 0-255 to -127 to 127
   sse_std_negbyt=where(sse_std ge 128)
   sse_std_sgnbyte=sse_std
   sse_std_sgnbyte(sse_std_negbyt)=sse_std_sgnbyte(sse_std_negbyt)-255

   ;sse_std_bad=where(sse_std_sgnbyte eq sse_std_min,cbsse_std) 
   sse_stdf=sse_std_scale_factor*sse_std_sgnbyte+sse_std_add_offset
   ;sse_stdf(sse_std_bad)=nfill 
   sse_stdf=transpose(sse_stdf)
   ;pimage,sse_stdf,mis_val=nfill,mis_color=1,vmin=0.5,vmax=1.2,$
   ;   /ps_port,/encapsulated,ps_file='amsr_std.eps',title='STD',units='(Kelvin)'

   ;******************************
   ; QC Flags:  OJO: BITS! 
   ;******************************
   ; rejection flag: 0.SST out of range, 1.cosmetic val, 2.IR_cloud, 3.MW_rain,
   ; 4.MW_ice, 5.MW_wind, 6.Land, 7.Spare
   flag_reject=transpose(reform(rejection_flag))
   ;pimage,flag_reject,mis_val=nfill,mis_color=1,$
   ;   /ps_port,/encapsulated,ps_file='amsr_rejflg.eps',title='Rejection Flag',units='(Bits)'

   ; confidence flag: 0.Side lobe contamination, 1.relaxed rain contam, 
   ; 2.TMI SST retrieved in SST, 3.High wnd,
   ; 4.seaice retrieved for MW, 5.sun_glint, 6.native bias and std
   flag_conf=transpose(reform(confidence_flag))
   ;pimage,flag_conf,mis_val=nfill,mis_color=1,$
   ;   /ps_port,/encapsulated,ps_file='amsr_conflg.eps',title='Confidence Flag',units='(Bits)'

   ;  Proximity Confidence is empty
   proxconf=transpose(reform(proximity_confidence))
   ;pimage,proxconf,mis_val=nfill,mis_color=1

   ;******************************
   ; Extracting Latitude and Longitude 
   ;******************************
   latf=transpose(lat)   
   lat_nfill=long(strcompress(lat__fillvalue,/remove_all))
   ;latbad=where(latf lt -90 or latf gt 90 or latf eq -9999.0,clatb)
   latbad=where(latf eq lat_nfill,clatb)
   latf(latbad) = nfill
   ;pimage,latf,mis_val=nfill,mis_color=1,vmin=-90,vmax=90,$
   ;   /ps_port,/encapsulated,ps_file='amsr_lat.eps',title='Latitude',units='(Degrees North)'

   lonf=transpose(lon)
   lon_nfill=long(strcompress(lon__fillvalue,/remove_all))
   lonbad=where(lonf eq lon_nfill,clonb)
   lonf(lonbad) = nfill
   ;pimage,lonf,mis_val=nfill,mis_color=1,vmin=-180,vmax=180,$
   ;   /ps_port,/encapsulated,ps_file='amsr_lon.eps',title='Longitude',units='(Degrees East)'

   ;******************************
   ; Extracting (nj,ni) positions: i (row,swath, or scan line), j (column or spot) 
   ;******************************
   nj=n_elements(sstf(*,0))
   ni=n_elements(sstf(0,*))
   ival=intarr(ni*nj)
   jval=intarr(ni*nj)
   cnt=0L
   for i=0,ni-1 do begin
     for j=0,nj-1 do begin
        ival(cnt)=i
        jval(cnt)=j
        cnt=cnt+1L
     endfor
   endfor 

   ;******************************
   ; Saving data into a binary structure 
   ;******************************
   ;retrvsst=where(sstf ne nfill and flag_reject ne 3 and flag_reject ne 6,crsst)
   retrvsst=where(sstf ne nfill,crsst)

   A={tme:dblarr(1),line:intarr(1),spot:intarr(1),lat:fltarr(1),lon:fltarr(1),sst:fltarr(1),sst_bias:fltarr(1),sst_std:fltarr(1),wspd:fltarr(1),dtwarm:fltarr(1),rej_flg:bytarr(1),conf_flg:bytarr(1),prox_flg:bytarr(1)}

   All=replicate(A,crsst)
 
   for i=0L,crsst-1L do begin
     All(i).tme=jultme(retrvsst(i))
     All(i).line=ival(retrvsst(i))
     All(i).spot=jval(retrvsst(i))
     All(i).lat=latf(retrvsst(i))
     All(i).lon=lonf(retrvsst(i))
     All(i).sst=sstf(retrvsst(i))
     All(i).sst_bias=sse_biasf(retrvsst(i))
     All(i).sst_std=sse_stdf(retrvsst(i))
     All(i).wspd=wndf(retrvsst(i))
     All(i).dtwarm=dtwf(retrvsst(i))
     All(i).rej_flg=flag_reject(retrvsst(i))
     All(i).conf_flg=flag_conf(retrvsst(i))
     All(i).prox_flg=proxconf(retrvsst(i))
   endfor

   name='../../../bin/2004/jan_2004/'+filestr(k)
   openw,unit,name,/get_lun
   writeu,unit,All
   close,unit
   free_lun,unit

endfor


;***********************************************
;   Testing saving image
;***********************************************
;dum=fltarr(nj,ni)
;dum[*,*]=nfill
;for k=0L,crsst-1L do dum(jval(retrvsst(k)),ival(retrvsst(k)))=sstf((retrvsst(k)))
;pimage,dum,mis_val=nfill,mis_color=1,vmin=280,vmax=305

stop
end
