;---------------------------------------------------------------------------
; NAME: nearest_index
;
; PURPOSE:   Finds the index in a vector whose value is closest to specified input
;            value.
;
; INPUTS:
;         vec...........input vector (monotomically increasing or decreasing)
;         inval.........value to test against vector values
;
; INPUT KEYWORDS:
;         /nomessage....no output message regarding out of bounds values
;         /endpoints....Data outside boundaries put in boundary endpoints
;
; FUNCTION OUTPUT:
;         index......index value of vector vec where inval is closest
;
; CREATION HISTORY:
;                   Darren Jackson CIRES/ETL 1998
;                                  Last updated March 2003
;---------------------------------------------------------------------------

FUNCTION nearest_index, vec, inval,nomessage=nomessage,endpoints=endpoints

index=-1
n=N_ELEMENTS(vec)
IF(n gt 1 ) THEN BEGIN
   IF(KEYWORD_SET(endpoints)) THEN BEGIN
      newdiff = ABS(vec - inval)
      mindiff = MIN(newdiff,index)
   ENDIF ELSE BEGIN
     IF(inval ge min(vec) and inval le max(vec)) THEN BEGIN
        newdiff = ABS(vec - inval)
        mindiff = MIN(newdiff,index)
     ENDIF ELSE BEGIN
       IF(not KEYWORD_SET(nomessage)) THEN BEGIN
         print,'Input value is not with vector range: Index set to -1'
         print,'Input value = ',inval
         print,'Maximum vector value = ',max(vec)
         print,'Minimum vector value = ',min(vec)
       ENDIF
     ENDELSE
   ENDELSE
ENDIF ELSE BEGIN
   PRINT,'Vector insufficient length: length = ',n
   PRINT,'Index set to -1'
ENDELSE

RETURN, index
end