;**************************************************************
;*
;* PURPOSE: To read a file that is mixed fltpoints and strings
;*
;* Reads the buoyYYMM.out files that result from running ext_buoy_region
;* This programs extracts the buoys that seem to be given good SSTs 
;* Area of interes= 65N 170E to 75N 120W
;* CDC DATA FORMAT
;* 0: YEAR (2002) 1: month  2: day of month
;* 3: hour 4: lat 5: lon 6: SST 7: BUOY ID
;**************************************************************
;**************************************************************
@library.pro     ;library should include: hhmm2fractime & yymmdd2jtimearrmod

;* files
in=0L
tot=20000L              ;use a big number for a temporary array
tcol=7                  ;total number of columns in file to be read
ncol=4                  ;number of columns needed in final array
data=dblarr(ncol,tot)
;norlat=75.0
;soulat=65.0
;easlon=240.0
;weslon=170.0

spawn,('ls -1 buoy02*.out'),file_list   ;
print,file_list
totfiles=n_elements(file_list)
filestr=strarr(totfiles)
for i=0,totfiles-1 do begin
   print,file_list(i)
   filestrdum=string(file_list(i),"(A19)")
   temp=str_sep(filestrdum,'.')
   filestr(i)=strcompress(temp(0))
   print,filestr
   ;******************************
   ; opening file
   ;******************************
   openr,unit,file_list(i),/get_lun
   numlines='wc '+ file_list(i)
   spawn,numlines,dum
   reads,dum,format='(I7)',nuln
   lns=strcompress(nuln)
   print,lns
   ;******************************
   ;*  reading file
   ;*  This file has 7 columns with numbers and the last one with letters
   ;*  Needs to read array as a STRARR, then separate the fields
   ;*  NOTE: STRSPLIT does the same as STR_SEP but it is smarter!!
   ;******************************
   d=fltarr(tcol,lns)                       ;create temporary array
   ID=strarr(lns)
   dstr=strarr(lns)
   readf,unit,dstr
   close,unit
   free_lun,unit
   for j=0,lns-1 do begin
      dum=strsplit(strtrim(dstr(j)),/extract) 
      d(*,j)=dum(0:6) 
      ID(j)=dum(7)
   endfor
   ;******************************
   ;* Selecting data corresponding to buoy ID: DIXX
   ;* Only one that gives reasonable SSTs
   ;******************************
   idx = where(ID eq 'DIXX',ct) 
   print,ct
   ;******************************
   ; saving data into array
   ;******************************
   if ct gt 0 then begin
      min=fltarr(ct)  ;no minutes given
      ltot=ct+in
      dnew=d(*,idx)
      fraction_day=hhmm2fractime(dnew(3,*),min)    ;hour,minute
      data(0,in:ltot-1)=yymmdd2jtimearrmod(dnew(1,*),dnew(2,*),fraction_day)  ;Julday mm,dd+%day
      data(1,in:ltot-1)=dnew(4,*)            ;Latitude
      data(2,in:ltot-1)=dnew(5,*)            ;Longitude
      data(3,in:ltot-1)=dnew(6,*)+273.15     ;SST originally in Celsi
      in=ltot
    endif
endfor
data=reform(data(*,0:ltot-1))
;******************************
; saving read data
;******************************
;openw,unit,'finalbuoy02.dat',/get_lun
;for i=0L,ltot-1L do printf,unit,format='(f10.5,2x,f10.6,2x,f10.5,2x,f6.2)',data(0,i),data(1,i),data(2,i),data(3,i)
;close,unit
;free_lun,unit
;******************************
in=0
ltot=0
print,'Done !'
stop
end

