PRO stitch_MODISA_SST_granules_25k,ncdir,dayname

;***********************************************
;*
;*   This program reads MODIS L2CORE SST granules and grids them in 
;*   a 0.25 deg x 0.25 deg global grid corresponding to the AMSRE grid 
;*   It saves the average, the average of the squares, and the count of each grid cell
;*   needed for further composite of subscale variability
;*
;*********************************************************
;!path = '~/idl_graphics:' + !path

;* test code outside script:
;ncdir='/projects/goes_buoy/MODIS_A/L2_DATA/2007/'
;dayname='20070101'

nfill= -999.0

;  Define new grid
xdim=1440. 
ydim=720. 

lon_slope = xdim/360.
lon_c = 180 * lon_slope
lat_slope = ydim/180.
lat_c = 90 * lat_slope          ; 90N to 90S
dlon=1/lon_slope
dlat=1/lat_slope
lat_25k=-90+dlat/2.+findgen(ydim)*dlat
lon_25k=-180+dlon/2.+findgen(ydim)*dlon

ngtcnt=fltarr(xdim,ydim)
daycnt=fltarr(xdim,ydim)
dayvaltme=replicate(nfill,xdim,ydim)
ngtvaltme=replicate(nfill,xdim,ydim)
dayvalsst=replicate(nfill,xdim,ydim)
ngtvalsst=replicate(nfill,xdim,ydim)
dayvalsstsqr=replicate(nfill,xdim,ydim)
ngtvalsstsqr=replicate(nfill,xdim,ydim)

spawn,('ls -1 '+ncdir+dayname+'*.nc'),file_list   ;NetCDF data
ndays=n_elements(file_list)
filestr=strarr(ndays)

for k=0,ndays-1 do begin

    ;print,file_list(k)
    temp=str_sep(file_list(k),'/')
    temp2=str_sep(temp(6),'-')
    fdate=temp2(0)

    year=strmid(fdate,0,4)
    srtmonth=strmid(fdate,4,2)
    srtday=strmid(fdate,6,2)
    mmdd=strmid(fdate,4,4)
    mmname=monthnum2monthstr_v1(srtmonth)
    month_dir=mmname+'_'+year
    ;jday=calday2jday(fix(year),fix(srtmonth),fix(srtday))
    ;stop

    ;******************************
    ; opening file
    ; To generate the script that reads the NetCDF, run the following code:
    ;******************************
    ;cdf2idl,file_list(k),outfile='read_NetCDF_MODISA',verbose='verbose'
    ;stop

    ;******************************
    ncid = NCDF_OPEN(file_list(k))
    @read_NetCDF_MODISA
    ;stop

    ;******************************
    ; Extracting Latitude and Longitude 
    ;******************************
    maxlat=float(strcompress(northernmost_latitude,/remove_all))
    minlat=float(strcompress(southernmost_latitude,/remove_all))
    maxlon=float(strcompress(easternmost_longitude,/remove_all))
    minlon=float(strcompress(westernmost_longitude,/remove_all))
 
    ;****************************** 
    ;* MTSAT SST (Kelvin): 214x108
    ;******************************
    scale_factor=float(strcompress(SEA_SURFACE_TEMPERATURE_SCALE_FACTOR,/remove_all))
    add_offset=float(strcompress(SEA_SURFACE_TEMPERATURE_add_offset,/remove_all))
    sst_min=fix(strcompress(SEA_SURFACE_TEMPERATURE_valid_min,/remove_all))
    sst_max=fix(strcompress(SEA_SURFACE_TEMPERATURE_valid_max,/remove_all))
    sst_fill=long(strcompress(SEA_SURFACE_TEMPERATURE__fillvalue,/remove_all))
    sst=SEA_SURFACE_TEMPERATURE
    sst_bad=where(sst lt sst_min or sst gt sst_max or sst eq sst_fill,cbsst)
    sstf=scale_factor*sst;+add_offset
    sstf(sst_bad)=nfill

    ;******************************
    ;* Proximity confidence (1-2-3-4-5):1600x800X24: Use values >3 (4-5 optimum), 3: possible cloud contamination?
    ;******************************
    conflag=PROXIMITY_CONFIDENCE

    ;window,2
    ;pimage,conflag,minlat=minlat,maxlat=maxlat,minlon=minlon,maxlon=maxlon,mis_val=nfill,mis_color=1,vmin=1,vmax=6
    ;stop

    ; Mascara nube <=2
    qc=where(conflag lt 3,cqc)
    sstf(qc)=nfill
    sstok=where(sstf ne nfill,cok)

    ;  Identify valid day and nighttime points (ascending pass = day; descending pass = ngt)
    ;  Checking slope in latitudes to see the sign of the tan(angle)
    ;  if angle is positve: ascending = day; else negative: descending:ngt)
    
    sizear=size(lat)
    latsize=sizear(2)
    slope=(lat(0,latsize-1)-lat(0,0))/latsize
    angle=atan(slope)*!radeg

    ;  Passing from L2 (data in scan line x scan spot) to L3 (gridding to 25 deg x 25 deg)
    ;  Convert lat and lon to grid coordinates 

    lonidx=where(lon ge 180.0,clon)
    if clon gt 0 then lon(lonidx) = lon(lonidx) - 360.0

    lon_c = -180 * lon_slope
    lat_c = -90 * lat_slope          ; 90N to 90S
    igrid = fix (lon * lon_slope - lon_c )
    jgrid = fix ((lat * lat_slope) - lat_c)

    ;  Fill in grids for day and night separately 

    case 1 of (angle gt 0): Begin
        for i=0L,cok-1L do begin
          ii = igrid(sstok(i))
          jj = jgrid(sstok(i))
          if ii ge 1440 then ii = ii - 1440.
          newcnt = daycnt(ii,jj) + 1L
          dayvalsst(ii,jj) = (dayvalsst(ii,jj)*float(daycnt(ii,jj)) + sstf(sstok(i))) / float(newcnt)
          dayvalsstsqr(ii,jj) = (dayvalsstsqr(ii,jj)*float(daycnt(ii,jj)) + (sstf(sstok(i))))^2 / float(newcnt)
          daycnt(ii,jj) = newcnt
        endfor

        ;window,0
        ;pimage,dayvalsst,mis_val=nfill,mis_color=1,coast=2,ccolor=2,$
        ;minlat=-90,maxlat=90,minlon=-180,maxlon=180,$
        ;title='MODISA SST for Daytime '+fdate,units='(deg C)';,$
        ;;/ps_port,/encapsulated,ps_file='k10_sst.eps'

    end
    else: Begin 
        for i=0L,cok-1L do begin
          ii = igrid(sstok(i))
          jj = jgrid(sstok(i))
          if ii ge 1440 then ii = ii - 1440.
          newcnt = ngtcnt(ii,jj) + 1L
          ngtvalsst(ii,jj) = (ngtvalsst(ii,jj)*float(ngtcnt(ii,jj)) + sstf(sstok(i))) / float(newcnt)
          ngtvalsstsqr(ii,jj) = (ngtvalsstsqr(ii,jj)*float(ngtcnt(ii,jj)) + (sstf(sstok(i))))^2 / float(newcnt)
          ngtcnt(ii,jj) = newcnt
        endfor

        ;window,2
        ;pimage,ngtvalsst,mis_val=nfill,mis_color=1,coast=2,ccolor=2,$
        ;minlat=-90,maxlat=90,minlon=-180,maxlon=180,$
        ;title='MODISA SST for Nighttime '+fdate,units='(deg C)';,$
        ;;/ps_port,/encapsulated,ps_file='k10_sst.eps'

     endelse
    endcase

    ;***  Save data into daily arrays ***
    ;std formula = sqrt((SUM[x^2] - SUM[x]^2)/n / (n-1))

    mean_file_d='../../L3_DATA/'+month_dir+'/mean_modis_'+year+'_'+mmdd+'.day.gz'
    openw,out,mean_file_d,/get_lun,/compress
    writeu,out,dayvalsst
    close,out
    free_lun,out

    meansqr_file_d='../../L3_DATA/'+month_dir+'/mnsqr_modis_'+year+'_'+mmdd+'.day.gz'
    openw,out,meansqr_file_d,/get_lun,/compress
    writeu,out,dayvalsstsqr
    close,out
    free_lun,out

    cnt_file_d='../../L3_DATA/'+month_dir+'/cnt_modis_'+year+'_'+mmdd+'.day.gz'
    openw,out,cnt_file_d,/get_lun,/compress
    writeu,out,daycnt
    close,out
    free_lun,out

    mean_file_n='../../L3_DATA/'+month_dir+'/mean_modis_'+year+'_'+mmdd+'.ngt.gz'
    openw,out,mean_file_n,/get_lun,/compress
    writeu,out,ngtvalsst
    close,out
    free_lun,out

    meansqr_file_n='../../L3_DATA/'+month_dir+'/mnsqr_modis_'+year+'_'+mmdd+'.ngt.gz'
    openw,out,meansqr_file_n,/get_lun,/compress
    writeu,out,ngtvalsstsqr
    close,out
    free_lun,out

    cnt_file_n='../../L3_DATA/'+month_dir+'/cnt_modis_'+year+'_'+mmdd+'.ngt.gz'
    openw,out,cnt_file_n,/get_lun,/compress
    writeu,out,ngtcnt
    close,out
    free_lun,out

    print,'Done with ngttime ',file_list(k)

endfor
;stop

end


