pro write_sst_netcdf, monthdir, date, lon_arr, lat_arr, mask_arr, mtsat_arr, ramssa_arr

;----------------------------------------------------------------------------
; Open ACCESS-R netcdf file in given directory (dir) for required date and hour (YYYYMMDDHH) 
; and convert to required TWP+ experiment NWP file format (v2) for date.
; Output filename = YYYYMMDD_TWP_DW_experiment_v2_NWP_ACCESS-R.nc
;
; Author: Helen Beggs, 21 Apr 2011
; Modified:
; 01 18 May 2011 hmb	Modified for operational ACCESS-R - changed land mask name.
; 02 14 Sep 2011 slc	Modified for MTSAT SSTf regridded to ACCESS-R 
; OJO! THis one is not called.  Is the one in the library!!!!!!!!!!!!!!!
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; Calculate the date - 1 day (prevdate)
;----------------------------------------------------------------------------

print, 'Current Date = ', date
int_date = long(date)
yyyy = fix(int_date/10000L)			; Integer year
mm = fix((int_date - yyyy*10000L)/100L)		; Integer month
dd = fix(int_date - yyyy*10000L - mm*100L)	; Integer day of month
prev_dd = dd-1					; Integer day of month

error = 0
julday1 = julday(mm,dd,yyyy,0,0,0)	; Convert date to Julian Day

current_date_time = SYSTIME(/JULIAN)
caldat, current_date_time, month, day, year

separator = "-"
creation_date = STRTRIM(year,2)+"-"+STRTRIM(month,2)+"-"+STRTRIM(day,2)
local_time_date = strcompress(string(yyyy)+separator+string(mm)+separator+string(dd),/remove_all)
utc_date = strcompress(string(yyyy)+separator+string(mm)+separator+string(prev_dd),/remove_all)
start_time = "11:44 (UTC)"
stop_time = "23:44 (UTC)"

;stop
;----------------------------------------------------------------------------
; Find files for date 0 UT run.
;----------------------------------------------------------------------------

;files = FindFile(dir+'/accessr_'+date+'0000', Count=found)

;if (found lt 1) then begin
;  print, 'ERROR: No ACESSR-R file found for date ', date
;  goto, finish
;endif

;----------------------------------------------------------------------------
; Read ACCESS-R netcdf file
;----------------------------------------------------------------------------

;infile = files(0)
;print, infile
;cdfid = ncdf_open(infile)
;dateid = ncdf_varid(cdfid,'valid_date')
;timeid = ncdf_varid(cdfid,'valid_time')
;lonid = ncdf_varid(cdfid,'lon')
;latid = ncdf_varid(cdfid,'lat')
;; Experimental ACCESS-R
;; maskid = ncdf_varid(cdfid,'sea_lnd_ice_mask') ; Mask for surface.  0 = sea, 1 = land, 2 = sea-ice
;; Operational ACCESS-R
;maskid = ncdf_varid(cdfid,'lnd_mask')  		; Mask for surface.  0 = sea, 1 = land, 2 = sea-ice
;uwndid = ncdf_varid(cdfid,'u10')   	   	; Instantaneous hourly zonal 10 m wind speed
;vwndid = ncdf_varid(cdfid,'v10')	 	; Instantaneous hourly meridional 10 m wind speed
;avuwndid = ncdf_varid(cdfid,'av10u')      	; Mean hourly zonal 10 m wind speed
;avvwndid = ncdf_varid(cdfid,'av10v')	; Mean hourly meridional 10 m wind speed
;avnetssiid = ncdf_varid(cdfid,'av_netswsfc')  	; Mean net shortwave radiation at surface
;avnetdliid = ncdf_varid(cdfid,'av_netlwsfc')  	; Mean net longwave radiation at surface
;fricvelid = ncdf_varid(cdfid,'fric_vel')	; Instantaneous friction velocity
;sensheatid = ncdf_varid(cdfid,'sens_hflx') 	; Instantaneous surface sensible heat flux (pos up)
;latheatid = ncdf_varid(cdfid,'lat_hflx')  	; Instantaneous surface latent heat flux (pos up)
;; avsensheatid = ncdf_varid(cdfid,'av_sens_hflx') ; Mean surface sensible heat flux (pos up)
;avlatheatid = ncdf_varid(cdfid,'av_lat_hflx')  	; Mean surface latent heat flux (pos up)
;ustressid = ncdf_varid(cdfid,'uwnd_strs')      	; Instantaneous hourly zonal surface wind stress
;vstressid = ncdf_varid(cdfid,'vwnd_strs')	; Instantaneous hourly meridional surface wind stress
;avustressid = ncdf_varid(cdfid,'av_uwnd_strs')  ; Mean hourly zonal surface wind stress
;avvstressid = ncdf_varid(cdfid,'av_vwnd_strs')	; Mean hourly meridional surface wind stress
;sstid = ncdf_varid(cdfid,'sfc_temp')		; Surface temperature - over the ocean from RAMSSA
;precipid = ncdf_varid(cdfid,'accum_prcp')	; Accumulated hourly precipitation
;; hr24precipid = ncdf_varid(cdfid,'hr24_prcp')	; Accumulated 24 hourly precipitation
;pressureid = ncdf_varid(cdfid,'sfc_pres')	; Instantaneous hourly surface pressure
;screentempid = ncdf_varid(cdfid,'temp_scrn')	; Instantaneous hourly screen temperature
;humidityid = ncdf_varid(cdfid,'av_qsair_scrn')	; Mean hourly specific humidity

;ncdf_varget, cdfid, dateid, date_arr
;ncdf_varget, cdfid, timeid, time_arr
;ncdf_varget, cdfid, lonid, lon_arr
;ncdf_varget, cdfid, latid, lat_arr
;ncdf_varget, cdfid, maskid, mask_arr
;ncdf_varget, cdfid, uwndid, uwind_arr
;ncdf_varget, cdfid, vwndid, vwind_arr
;;ncdf_varget, cdfid, avuwndid, av_uwind_arr
;ncdf_varget, cdfid, avvwndid, av_vwind_arr
;ncdf_varget, cdfid, avnetssiid, av_netssi_arr
;ncdf_varget, cdfid, avnetdliid, av_netdli_arr
;ncdf_varget, cdfid, fricvelid, fricvel_arr
;ncdf_varget, cdfid, sensheatid, sensheat_arr
;ncdf_varget, cdfid, latheatid, latheat_arr
;; ncdf_varget, cdfid, avsensheatid, av_sensheat_arr
;ncdf_varget, cdfid, avlatheatid, av_latheat_arr
;ncdf_varget, cdfid, ustressid, ustress_arr
;ncdf_varget, cdfid, vstressid, vstress_arr
;ncdf_varget, cdfid, avustressid, av_ustress_arr
;ncdf_varget, cdfid, avvstressid, av_vstress_arr
;ncdf_varget, cdfid, sstid, surf_temp_arr
;ncdf_varget, cdfid, precipid, acc_precip_arr
;; ncdf_varget, cdfid, hr24precipid, acc_24hrprecip_arr
;ncdf_varget, cdfid, pressureid, surf_press_arr
;ncdf_varget, cdfid, screentempid, screen_temp_arr
;ncdf_varget, cdfid, humidityid, humidity_arr

;ntimeid = ncdf_dimid(cdfid, 'time')
;nlonid = ncdf_dimid(cdfid, 'lon')
;nlatid = ncdf_dimid(cdfid, 'lat')
;ncdf_diminq, cdfid, ntimeid, name, ntime
;ncdf_diminq, cdfid, nlonid, name, nlon
;ncdf_diminq, cdfid, nlatid, name, nlat

;print, 'sze of time = ', ntime

nlon=n_elements(lon_arr)
nlat=n_elements(lat_arr)

print, 'size of lon_arr = ', nlon
print, 'size of lat_arr = ', nlat

print, 'Min lon = ', lon_arr(0), ', Max lon = ', lon_arr(nlon-1)
print, 'Min lat = ', lat_arr(nlat-1), ', Max lat = ', lat_arr(0)

print, 'size of mask_arr = ', size(mask_arr, /DIMENSIONS)

;-----------------------------------------------------------------------------
; Convert float arrays to byte or short integer (2 bytes) arrays
;-----------------------------------------------------------------------------

;landmask_temp = bytarr(nlon,nlat,ntime)
;landmask_temp = landmask_temp - 128		; Fill values
;U_wind_10m_arr = intarr(nlon,nlat,ntime)
;U_wind_10m_arr = U_wind_10m_arr - 32768		; Fill values
;sst_arr = U_wind_10m_arr
;precip_arr = U_wind_10m_arr

;temperature_arr = U_wind_10m_arr
;spec_hum_arr = U_wind_10m_arr

;mask_0 = WHERE((mask_arr GE 0) AND (mask_arr LE 0.5), count)	; 0 = sea
;mask_1 = WHERE((mask_arr GT 0.5) AND (mask_arr LE 2), count)	; 1 = land
;if (count EQ 0) then print, 'No land in mask_arr!'
;landmask_temp(mask_0) = 0
;landmask_temp(mask_1) = 1
;landmask_arr = bytarr(nlon,nlat)
;for i = 0, nlon - 1 do begin
;  for j = 0, nlat - 1 do begin
;;    landmask_arr(i,j) = landmask_temp(i,j,0)
;    landmask_arr(i,j) = landmask_temp(i,j,ntime - 1)
;  endfor
;endfor
;index = WHERE(landmask_arr EQ 1, count)
;if (count EQ 0) then print, 'No land in landmask_arr!'


;temperature_index = WHERE(screen_temp_arr GT -1000000.)
;temperature_arr(temperature_index) = fix(screen_temp_arr(temperature_index) * 100.)
;humidity_index = WHERE(humidity_arr GT -1000000.)
;spec_hum_arr(humidity_index) = fix(humidity_arr(humidity_index) * 100000.)

nfill=-999.990
mtsat_arr_short = intarr(nlon,nlat)- 32768
ramssa_arr_short = mtsat_arr_short

mtsat_index = where(mtsat_arr ne nfill)
mtsat_arr_short(mtsat_index) = fix(mtsat_arr(mtsat_index) * 100.)
ramssa_index = where(ramssa_arr ne nfill)
ramssa_arr_short(ramssa_index) = fix(ramssa_arr(ramssa_index) * 100.)

;-----------------------------------------------------------------------------
; Now copy new arrays to new netCDF file
;-----------------------------------------------------------------------------

outdir = '/projects/ftp/pub/sandrac/helen/'+monthdir
;outfile = outdir + '/' + date + '_TWP+DW_experiment_v1_MTSAT-1R_SSTfnd_regrid.nc'  
outfile = outdir + '/' + date + '_TWP_DW_experiment_v1_MTSAT-1R_SSTfnd_regrid.nc'  
newcdfid = ncdf_create(outfile,/CLOBBER)

;ntime2id = ncdf_dimdef(newcdfid, 'date', /UNLIMITED)
nlat2id = ncdf_dimdef(newcdfid, 'lat', nlat)
nlon2id = ncdf_dimdef(newcdfid, 'lon', nlon)

lat2id = ncdf_vardef(newcdfid, 'lat', [nlat2id],/FLOAT)
lon2id = ncdf_vardef(newcdfid, 'lon', [nlon2id],/FLOAT)
mask2id = ncdf_vardef(newcdfid, 'landmask', [nlon2id,nlat2id],/BYTE)
mtsstf2id = ncdf_vardef(newcdfid, 'mtsat_sstf', [nlon2id,nlat2id],/SHORT)
ramsstf2id = ncdf_vardef(newcdfid, 'ramssa_sstf', [nlon2id,nlat2id],/SHORT)

;sst2id = ncdf_vardef(newcdfid, 'surface_temperature', [nlon2id,nlat2id,ntime2id],/SHORT)

ncdf_control, newcdfid, /ENDEF 		; Put netCDF file into data mode
ncdf_varput, newcdfid, lat2id, lat_arr 
ncdf_varput, newcdfid, lon2id, lon_arr
ncdf_varput, newcdfid, mask2id, mask_arr
ncdf_varput, newcdfid, mtsstf2id, mtsat_arr_short
ncdf_varput, newcdfid, ramsstf2id, ramssa_arr_short

;ncdf_varput, newcdfid, date2id, date
;ncdf_varput, newcdfid, time2id, time_arr
;;ncdf_varput, newcdfid, sst2id, sst_arr

;-------------------------------------------------------------------------------------
; Copy some variable attributes to new netCDF file
;-------------------------------------------------------------------------------------

;ncdf_control, newcdfid, /REDEF		; Put netCDF file into define mode
    
;n = ncdf_varinq(cdfid,latid)
;for i=0,n.NATTS-1 do begin
;  result = ncdf_attcopy(cdfid, latid, ncdf_attname(cdfid,latid,i), newcdfid, lat2id)
;endfor
;n = ncdf_varinq(cdfid,lonid)
;for i=0,n.NATTS-1 do begin
;  result = ncdf_attcopy(cdfid, lonid, ncdf_attname(cdfid,lonid,i), newcdfid, lon2id)
;endfor 
;n = ncdf_varinq(cdfid,timeid)
;for i=0,n.NATTS-1 do begin
;  result = ncdf_attcopy(cdfid, dateid, ncdf_attname(cdfid,dateid,i), newcdfid, date2id)
;endfor 
;for i=0,n.NATTS-1 do begin
;  result = ncdf_attcopy(cdfid, timeid, ncdf_attname(cdfid,timeid,i), newcdfid, time2id)
;endfor 

;-------------------------------------------------------------------------------------
; Define new variable attributes to new netCDF file
;-------------------------------------------------------------------------------------
ncdf_control, newcdfid, /REDEF		; Put netCDF file into define mode
 
ncdf_attput,newcdfid,lat2id,'long_name','latitude'  
ncdf_attput,newcdfid,lon2id,'long_name','longitude'  

ncdf_attput,newcdfid,mask2id,'long_name','land_binary_mask'  
ncdf_attput,newcdfid,mask2id,'_FillValue',-128b
ncdf_attput,newcdfid,mask2id,'flag_values',[0b, 1b]    
ncdf_attput,newcdfid,mask2id,'flag_meanings','no_land land' 
ncdf_attput,newcdfid,mask2id,'source','ACCESS-R'   
ncdf_attput,newcdfid,mask2id,'comment','No interpolation - native ACCESS-R grid'  

ncdf_attput,newcdfid,mtsstf2id,'long_name','MTSAT-R1 Foundation SST'
ncdf_attput,newcdfid,mtsstf2id,'units','deg C' 
ncdf_attput,newcdfid,mtsstf2id,'_FillValue',-32768
ncdf_attput,newcdfid,mtsstf2id,'add_offset',0. 
ncdf_attput,newcdfid,mtsstf2id,'scale_factor',0.01
ncdf_attput,newcdfid,mtsstf2id,'valid_min',0
ncdf_attput,newcdfid,mtsstf2id,'valid_max',3600
ncdf_attput,newcdfid,mtsstf2id,'comment','re-gridded MTSAT SSTf to ACCESS-R grid.'

ncdf_attput,newcdfid,ramsstf2id,'long_name','RAMSSA Foundation SST'
ncdf_attput,newcdfid,ramsstf2id,'units','deg C' 
ncdf_attput,newcdfid,ramsstf2id,'_FillValue',-32768
ncdf_attput,newcdfid,ramsstf2id,'add_offset',0. 
ncdf_attput,newcdfid,ramsstf2id,'scale_factor',0.01
ncdf_attput,newcdfid,ramsstf2id,'valid_min',0
ncdf_attput,newcdfid,ramsstf2id,'valid_max',3600
ncdf_attput,newcdfid,ramsstf2id,'comment','re-gridded RAMSSA SSTf to ACCESS-R grid.'

;ncdf_attput,newcdfid,sst2id,'long_name','ACCESS-R surface temperature'
;ncdf_attput,newcdfid,sst2id,'units','K' 
;ncdf_attput,newcdfid,sst2id,'_FillValue',-32768
;ncdf_attput,newcdfid,sst2id,'add_offset',0. 
;ncdf_attput,newcdfid,sst2id,'scale_factor',0.01
;ncdf_attput,newcdfid,sst2id,'valid_min',-32767s  
;ncdf_attput,newcdfid,sst2id,'valid_max',32767s
;ncdf_attput,newcdfid,sst2id,'comment','No interpolation - native ACCESS-R grid.  Over the ocean corresponds to the most recently completed RAMSSA SSTfnd analysis.'

;ncdf_control, newcdfid, /ENDEF 		; Put netCDF file into data mode
;ncdf_varput, newcdfid, lat2id, lat_arr 
;ncdf_varput, newcdfid, lon2id, lon_arr
;ncdf_varput, newcdfid, mask2id, mask_arr
;ncdf_varput, newcdfid, mtsstf2id, mtsat_arr_short
;ncdf_varput, newcdfid, ramsstf2id, ramssa_arr_short


;-------------------------------------------------------------------------------------
; Define new global attributes to new netCDF file
;-------------------------------------------------------------------------------------

;current_date_time = SYSTIME(/JULIAN)
;caldat, current_date_time, month, day, year
;creation_date = STRTRIM(year,2)+"-"+STRTRIM(month,2)+"-"+STRTRIM(day,2)

;start_date_text = STRTRIM(date_arr(0),2)
;start_date = STRMID(start_date_text,0,4)+"-"+STRMID(start_date_text,4,2)+"-"+STRMID(start_date_text,6,2)
;stop_date_text = STRTRIM(date_arr(23),2)
;stop_date = STRMID(stop_date_text,0,4)+"-"+STRMID(stop_date_text,4,2)+"-"+STRMID(stop_date_text,6,2)
;if(time_arr(0)/100 LT 10) then begin
;  start_time = '0'+STRTRIM(time_arr(0)/100,2)+":00"
;endif else begin
;  start_time = STRTRIM(time_arr(0)/100,2)+":00"
;endelse
;if(time_arr(23)/100 LT 10) then begin
;  stop_time = '0'+STRTRIM(time_arr(23)/100,2)+":00"
;endif else begin
;  stop_time = STRTRIM(time_arr(23)/100,2)+":00"
;endelse

ncdf_attput, newcdfid, /GLOBAL, 'conventions', 'CF-1.0'
ncdf_attput, newcdfid, /GLOBAL, 'title', 'MTSAT-1R Foundation SST over TWP+ domain, ACCESS-R grid'
;ncdf_attput, newcdfid, /GLOBAL, 'references', ''
ncdf_attput, newcdfid, /GLOBAL, 'netcdf_version_id', '3.6.0'
ncdf_attput, newcdfid, /GLOBAL, 'creation_date', creation_date
ncdf_attput, newcdfid, /GLOBAL, 'local_time_date', local_time_date
ncdf_attput, newcdfid, /GLOBAL, 'utc_date', utc_date
;ncdf_attput, newcdfid, /GLOBAL, 'history', 'convert_twp_nwp.pro'
ncdf_attput, newcdfid, /GLOBAL, 'MTSAT_SSTf_date', local_time_date
ncdf_attput, newcdfid, /GLOBAL, 'RAMSSA_SSTfnd_date', utc_date
ncdf_attput, newcdfid, /GLOBAL, 'start_date', utc_date
ncdf_attput, newcdfid, /GLOBAL, 'start_time', start_time
ncdf_attput, newcdfid, /GLOBAL, 'stop_date', utc_date
ncdf_attput, newcdfid, /GLOBAL, 'stop_time', stop_time
ncdf_attput, newcdfid, /GLOBAL, 'spatial_resolution', '0.375 degree'
ncdf_attput, newcdfid, /GLOBAL, 'southernmost_latitude', lat_arr(0)
ncdf_attput, newcdfid, /GLOBAL, 'northernmost_latitude', lat_arr(nlat-1)
ncdf_attput, newcdfid, /GLOBAL, 'westernmost_longitude', lon_arr(0)
ncdf_attput, newcdfid, /GLOBAL, 'easternmost_longitude', lon_arr(nlon-1)
ncdf_attput, newcdfid, /GLOBAL, 'source_provider', 'University of Colorado'
ncdf_attput, newcdfid, /GLOBAL, 'contact', 'sandrac@olorado.edu'
ncdf_attput, newcdfid, /GLOBAL, 'comment', 'Outputs are MTSAT-1R SSTfnd and RAMSSA SSTfnd.  MTSAT SSTfnd is defined as the previous LOCAL night SST composite made of ordered SSTs from 22:00-23:00 pm (LST) from day before and 4:00-5:00 am (LST) from current day.  Only MTSAT PC5 quality data used.  Start date/time and stop/date/time are given in UTC. Corresponding RAMSSA SSTfnd is for previous UTC day consistent with start/stop UTC time.'

;ncdf_control, newcdfid, /ENDEF 		; Put netCDF file into data mode
;ncdf_varput, newcdfid, lat2id, lat_arr 
;ncdf_varput, newcdfid, lon2id, lon_arr
;ncdf_varput, newcdfid, mask2id, mask_arr
;ncdf_varput, newcdfid, mtsstf2id, mtsat_arr_short
;ncdf_varput, newcdfid, ramsstf2id, ramssa_arr_short

ncdf_close, newcdfid
;ncdf_close, cdfid
;
;finish:
;
end
